--------------------------------------------------------------------------------
-- Copyright (c) 1995-2003 Xilinx, Inc.
-- All Right Reserved.
--------------------------------------------------------------------------------
--   ____  ____ 
--  /   /\/   / 
-- /___/  \  /    Vendor: Xilinx 
-- \   \   \/     Version : 7.1.01i
--  \   \         Application : ISE Foundation
--  /   /         Filename : Testbench_Waveform.vhw
-- /___/   /\     Timestamp : Sun Apr 10 12:36:16 2005
-- \   \  /  \ 
--  \___\/\___\ 
--
--Command: 
--Design Name: Testbench_Waveform
--Device: Xilinx
--

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
USE IEEE.STD_LOGIC_TEXTIO.ALL;
USE STD.TEXTIO.ALL;

ENTITY Testbench_Waveform IS
END Testbench_Waveform;

ARCHITECTURE testbench_arch OF Testbench_Waveform IS
    COMPONENT top_level
        PORT (
            clk : In std_logic;
            reset : In std_logic;
            data_in : In std_logic_vector (7 DownTo 0);
            data_out : Out std_logic_vector (7 DownTo 0);
            load : In std_logic;
            externe_daten_in : In std_logic_vector (1 DownTo 0);
            control_out : Out std_logic_vector (1 DownTo 0)
        );
    END COMPONENT;

    SIGNAL clk : std_logic := '0';
    SIGNAL reset : std_logic := '1';
    SIGNAL data_in : std_logic_vector (7 DownTo 0) := "00000000";
    SIGNAL data_out : std_logic_vector (7 DownTo 0) := "00000000";
    SIGNAL load : std_logic := '0';
    SIGNAL externe_daten_in : std_logic_vector (1 DownTo 0) := "00";
    SIGNAL control_out : std_logic_vector (1 DownTo 0) := "00";

    SHARED VARIABLE TX_ERROR : INTEGER := 0;
    SHARED VARIABLE TX_OUT : LINE;

    constant PERIOD : time := 200 ns;
    constant DUTY_CYCLE : real := 0.5;
    constant OFFSET : time := 0 ns;

    BEGIN
        UUT : top_level
        PORT MAP (
            clk => clk,
            reset => reset,
            data_in => data_in,
            data_out => data_out,
            load => load,
            externe_daten_in => externe_daten_in,
            control_out => control_out
        );

        PROCESS    -- clock process for clk
        BEGIN
            WAIT for OFFSET;
            CLOCK_LOOP : LOOP
                clk <= '0';
                WAIT FOR (PERIOD - (PERIOD * DUTY_CYCLE));
                clk <= '1';
                WAIT FOR (PERIOD * DUTY_CYCLE);
            END LOOP CLOCK_LOOP;
        END PROCESS;

        PROCESS
            PROCEDURE CHECK_control_out(
                next_control_out : std_logic_vector (1 DownTo 0);
                TX_TIME : INTEGER
            ) IS
                VARIABLE TX_STR : String(1 to 4096);
                VARIABLE TX_LOC : LINE;
                BEGIN
                IF (control_out /= next_control_out) THEN
                    STD.TEXTIO.write(TX_LOC, string'("Error at time="));
                    STD.TEXTIO.write(TX_LOC, TX_TIME);
                    STD.TEXTIO.write(TX_LOC, string'("ns control_out="));
                    IEEE.STD_LOGIC_TEXTIO.write(TX_LOC, control_out);
                    STD.TEXTIO.write(TX_LOC, string'(", Expected = "));
                    IEEE.STD_LOGIC_TEXTIO.write(TX_LOC, next_control_out);
                    STD.TEXTIO.write(TX_LOC, string'(" "));
                    TX_STR(TX_LOC.all'range) := TX_LOC.all;
                    STD.TEXTIO.Deallocate(TX_LOC);
                    ASSERT (FALSE) REPORT TX_STR SEVERITY ERROR;
                    TX_ERROR := TX_ERROR + 1;
                END IF;
            END;
            PROCEDURE CHECK_data_out(
                next_data_out : std_logic_vector (7 DownTo 0);
                TX_TIME : INTEGER
            ) IS
                VARIABLE TX_STR : String(1 to 4096);
                VARIABLE TX_LOC : LINE;
                BEGIN
                IF (data_out /= next_data_out) THEN
                    STD.TEXTIO.write(TX_LOC, string'("Error at time="));
                    STD.TEXTIO.write(TX_LOC, TX_TIME);
                    STD.TEXTIO.write(TX_LOC, string'("ns data_out="));
                    IEEE.STD_LOGIC_TEXTIO.write(TX_LOC, data_out);
                    STD.TEXTIO.write(TX_LOC, string'(", Expected = "));
                    IEEE.STD_LOGIC_TEXTIO.write(TX_LOC, next_data_out);
                    STD.TEXTIO.write(TX_LOC, string'(" "));
                    TX_STR(TX_LOC.all'range) := TX_LOC.all;
                    STD.TEXTIO.Deallocate(TX_LOC);
                    ASSERT (FALSE) REPORT TX_STR SEVERITY ERROR;
                    TX_ERROR := TX_ERROR + 1;
                END IF;
            END;
            BEGIN
                -- -------------  Current Time:  1285ns
                WAIT FOR 1285 ns;
                load <= '1';
                -- -------------------------------------
                -- -------------  Current Time:  3085ns
                WAIT FOR 1800 ns;
                load <= '0';
                -- -------------------------------------
                -- -------------  Current Time:  4085ns
                WAIT FOR 1000 ns;
                reset <= '0';
                -- -------------------------------------
                -- -------------  Current Time:  5485ns
                WAIT FOR 1400 ns;
                load <= '1';
                -- -------------------------------------
                -- -------------  Current Time:  5885ns
                WAIT FOR 400 ns;
                load <= '0';
                -- -------------------------------------
                -- -------------  Current Time:  8485ns
                WAIT FOR 2600 ns;
                data_in <= "00001111";
                -- -------------------------------------
                -- -------------  Current Time:  9285ns
                WAIT FOR 800 ns;
                load <= '1';
                -- -------------------------------------
                -- -------------  Current Time:  9885ns
                WAIT FOR 600 ns;
                load <= '0';
                -- -------------------------------------
                WAIT FOR 90315 ns;

                IF (TX_ERROR = 0) THEN
                    STD.TEXTIO.write(TX_OUT, string'("No errors or warnings"));
                    ASSERT (FALSE) REPORT
                      "Simulation successful (not a failure).  No problems detected."
                      SEVERITY FAILURE;
                ELSE
                    STD.TEXTIO.write(TX_OUT, TX_ERROR);
                    STD.TEXTIO.write(TX_OUT,
                        string'(" errors found in simulation"));
                    ASSERT (FALSE) REPORT "Errors found during simulation"
                         SEVERITY FAILURE;
                END IF;
            END PROCESS;

    END testbench_arch;

