LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;
USE ieee.std_logic_textio.ALL;
USE std.textio.ALL;

ENTITY top_level_Testbench_vhd_tb IS
END top_level_Testbench_vhd_tb;

ARCHITECTURE behavior OF top_level_Testbench_vhd_tb IS 
	
	COMPONENT top_level
	PORT(
		clk : IN std_logic;
		reset : IN std_logic;
		data_in : IN std_logic_vector(7 downto 0);
		load : IN std_logic;          
		data_out : OUT std_logic_vector(7 downto 0)
		);
	END COMPONENT;

	SIGNAL clk :  std_logic;
	SIGNAL reset :  std_logic;
	SIGNAL data_in :  std_logic_vector(7 downto 0);
	SIGNAL data_out :  std_logic_vector(7 downto 0);
	SIGNAL load :  std_logic;

	CONSTANT clk_setup_time : time 				:= 3 ns;
	CONSTANT clk_period 		: time 				:= 40 ns;
	constant Version 			: string 			:= " Testbench V1.0a , Jan04  ";
	SIGNAL   Kommentar 	   : string(1 to 30) :="Null                          "; -- Bitte beachten: wenn Sie Strings verwenden, muss immer ein Bereich ( x to y ) angegeben werden. Der Bereich muss immer(!) voll angegeben werden; bei 1 to 30 mssen also immer Leerzeichen eingefgt werden, um auf insgesamt 30 Zeichen zu kommen. Als kleine Hilfe kann man sich eine "Maske" als Vergleich einfgen (siehe unten in der Testbench).

BEGIN

	uut: top_level PORT MAP(
		clk => clk,
		reset => reset,
		data_in => data_in,
		data_out => data_out,
		load => load
	);


-- *** Test Bench - User Defined Section ***
PROCESS -- clock process for clk,
	BEGIN
		CLOCK_LOOP : LOOP
 			clk <= transport '0';
 			WAIT FOR clk_setup_time;
			clk <= transport '1';
			WAIT FOR clk_period/2;
			clk <= transport '0';
			WAIT FOR (clk_period/2)-clk_setup_time;
		END LOOP CLOCK_LOOP;
	END PROCESS;
   
	tb : PROCESS
   BEGIN	
													-- Anmerkung: Report alleine (ohne ASSERT voran) funktioniert nur ab VHDL'93	
      REPORT Version;						-- Versions-Informationen ausgeben (rein zur Info des Users)
		REPORT Time'IMAGE(clk_period);	-- Time Konstante als Sting ausgeben
--		Das IMAGE Attribut besteht seit dem VHDL'93 Standard, um andere Typen in Strings zu wandeln. Achtung funktioniert nicht mit Vectoren!
--    Beispiel, um den Integer i als string zu reporten:
--    report Integer'image(i);
      ----------------------------------------------  <-"Lngen-Maske", um 30 Zeichen fr "Kommentar" zu erreichen= das Semikolen muss unter dem letzten Minus sitzen; sonst Fehlermeldung im Simulator
  		Kommentar <= "Reset Phase                   ";
		-------------------------------------------------------------------------
		data_in <= "10011001";
		load  <= '0';
		reset <= '1';
		WAIT FOR 10*clk_period;
		-------------------------------------------------------------------------
		ASSERT data_out="00000000"
      REPORT " ===== Fehler: data_out=0 erwartet ====="
      SEVERITY WARNING;
--		Syntax:
--		assert <condition>
--		report "<report_string>"
--		[severity {note|warning|error|failure}];
      ----------------------------------------------
		Kommentar <= "Normal zaehlen                ";
		-------------------------------------------------------------------------
		reset <= '0';
		WAIT FOR 20*clk_period;
		-------------------------------------------------------------------------
		ASSERT data_out="00010100"
      REPORT " ===== Fehler: data_out=20 erwartet ====="
      SEVERITY WARNING;
		-------------------------------------------------------------------------
		ASSERT data_out="00000000"
      REPORT " ===== Test: hier wird ein falsches Ergebnis vorgetuscht ====="
      SEVERITY WARNING;
		----------------------------------------------		
		Kommentar <= "Counter Laden mit Wert 153    ";
		-------------------------------------------------------------------------
		load  <= '1';
		WAIT FOR 4*clk_period;
		----------------------------------------------
		Kommentar <= "Counter zaehlt weiter         ";
		-------------------------------------------------------------------------
		load  <= '0';
		WAIT FOR 20*clk_period;
		reset <= '1';
		WAIT FOR 10*clk_period;
		----------------------------------------------
		Kommentar <= "Reset+Load gleichzeitig Test  ";
		-------------------------------------------------------------------------
		load  <= '1';
		WAIT FOR 10*clk_period;
		reset <= '0';
		WAIT FOR 1*clk_period;
		----------------------------------------------
		Kommentar <= "Load direkt hinter Reset Test ";
		-------------------------------------------------------------------------
		load  <= '0';
		WAIT FOR 50*clk_period;
		----------------------------------------------
		Kommentar <= "Testbench Ende                ";
		-------------------------------------------------------------------------
		WAIT FOR 20*clk_period;

		-- Zwei Mglichkeiten, die Testbench zu "beenden":
		-- Mit einer Meldung beenden
		ASSERT (FALSE) 
		REPORT "Simulation successful (not a failure)."
		SEVERITY FAILURE; -- FAILURE, um Abbruch zu provozieren
		-- Alternative: ewig weiterlaufen
		-- wait; -- will wait forever
   END PROCESS;
-- *** End Test Bench - User Defined Section ***

END;
