library IEEE;
use IEEE.std_logic_1164.all;

entity stopwatch is
    port (  CLK : in STD_LOGIC;
				RESET : in STD_LOGIC;
				STRTSTOP : in STD_LOGIC;
				TENTHSOUT : out STD_LOGIC_VECTOR(9 downto 0);
				ONESOUT : out STD_LOGIC_VECTOR(6 downto 0);
				TENSOUT : out STD_LOGIC_VECTOR(6 downto 0));
end stopwatch;

architecture inside of stopwatch is

component statmach
    port (  CLK : in STD_LOGIC;
          	RESET : in STD_LOGIC;
	       	STRTSTOP : in STD_LOGIC;
	       	CLKEN : out STD_LOGIC;
	     		RST : out STD_LOGIC);
end component;

-- Insert Coregen Counter Component Declaration.

component tenths
	port (
			Q: OUT std_logic_vector(3 downto 0);
			CLK: IN std_logic;
			Q_THRESH0: OUT std_logic;
			CE: IN std_logic;
			AINIT: IN std_logic);
end component;

component decode port (
			binary: in std_logic_vector(3 downto 0);
			one_hot: out std_logic_vector(9 downto 0));
end component;

component cnt60
    port (  CE : in STD_LOGIC;
			   CLK : in STD_LOGIC;
	   		CLR : in STD_LOGIC;
				LSBSEC : out STD_LOGIC_VECTOR(3 downto 0);
				MSBSEC : out STD_LOGIC_VECTOR(3 downto 0));
end component;

component hex2led
    port ( 	HEX : in STD_LOGIC_VECTOR(3 downto 0);
	   	 	LED : out STD_LOGIC_VECTOR(6 downto 0));
end component;

signal strtstopinv : STD_LOGIC;
signal clkenable : STD_LOGIC;
signal rstint : STD_LOGIC;
signal xcountout : STD_LOGIC_VECTOR(9 downto 0);
signal xtermcnt : STD_LOGIC;
signal cnt60enable : STD_LOGIC;
signal lsbcnt : STD_LOGIC_VECTOR(3 downto 0);
signal msbcnt : STD_LOGIC_VECTOR(3 downto 0);
signal Q: std_logic_vector(3 downto 0);

begin

MACHINE:statmach port map(CLK=>CLK,
	                  RESET=>RESET,
	                  STRTSTOP=>strtstopinv,
	                  CLKEN=>clkenable,
	               	RST=>rstint);

-- Insert Coregen Counter Instantiation.

XCOUNTER: tenths port map (
			Q => Q,
			CLK => CLK,
			Q_THRESH0 =>xtermcnt,
			CE => clkenable,
			AINIT => rstint);

decoder: decode port map (
			binary => Q,
			one_hot => xcountout);

sixty: cnt60 port map(CE=>cnt60enable,
							CLK=>CLK,
							CLR=>rstint,
							LSBSEC=>lsbcnt,
							MSBSEC=>msbcnt);

lsbled:hex2led port map(HEX=>lsbcnt,
							LED=>ONESOUT);

msbled:hex2led port map(HEX=>msbcnt,
					     LED=>TENSOUT);
			
cnt60enable<=xtermcnt and clkenable;
						  TENTHSOUT<=not(xcountout);
					     strtstopinv<=not(STRTSTOP);

end inside;

------------- Begin Cut here for CONFIGURATION snippet ------ CONF_TAG

-- synopsys translate_off

Library XilinxCoreLib;

 configuration cfg_tenths of stopwatch is
     for inside
 		for all : tenths use entity XilinxCoreLib.C_COUNTER_BINARY_V1_0(behavioral)
		generic map(
			c_sinit_val => "0",
			c_load_enable => 1,
			c_has_up => 0,
			c_sync_enable => 0,
			c_has_ainit => 1,
			c_sync_priority => 1,
			c_has_q_thresh1 => 0,
			c_has_q_thresh0 => 1,
			c_has_sinit => 0,
			c_thresh1_value => "1111111111111111",
			c_has_thresh1 => 0,
			c_has_thresh0 => 0,
			c_ainit_val => "0001",
			c_has_sset => 0,
			c_width => 4,
			c_load_low => 0,
			c_count_to => "1010",
			c_has_iv => 0,
			c_count_mode => 0,
			c_has_l => 0,
			c_has_aset => 0,
			c_has_sclr => 0,
			c_thresh0_value => "1010",
			c_pipe_stages => 0,
			c_has_load => 0,
			c_has_ce => 1,
			c_has_aclr => 0,
			c_count_by => "0001",
			c_restrict_count => 1,
			c_enable_rlocs => 1);
	end for;
  end for;
 end cfg_tenths;
-- synopsys translate_on


